#!/usr/bin/perl
######################################################
# 
# Retrieve the next MAC address.
#
# Copyright (C) 2003 nuBridges llc
# Released under the Gnu General Public License, Version 2
# Copyright (C) 2003  nuBridges llc
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: nextIP.pl,v 1.8 2003/11/13 21:41:12 cshapiro Exp $
# 
#####################################################
use strict;
use Getopt::Std;
use File::Basename;

use vars qw($opt_f $opt_t $opt_b $opt_o $opt_n $opt_i $opt_s);

# Hash ip_addres => name (or ip_addres => ip_address if name not known)
use vars qw(%IPNAME);



#
# Obligatory usage message.
#
sub usage($)
{
    print("Usage: $_[0] [-f filename] -t top -b bottom -n machine_name -s ip_address_string {-o {aqrd} -i ipaddress} \n");
    print<<EOU

	Print the next unused ip address from filename to stdout.  The
	"top" and "bottom" numbers if passed will limit the range of
	the ip address. Default is "1" and "254".  The "-D" argument
	allows you to manage the database of name/ip address
	pairs. The "-o"peration argument switch allows you to "a"dd,
	"r"emove, "q"uery, or "d"ump the database for an ip address
	("-i") or machine name ("-n"). When you get the next ip
	address, this program will loop through incrementing the
	current ip address and searching the database for it until it
	finds an unused address to return; it will add this address
	and machine name to the database. When you are finished with
	an ip address, you must call this program with the "-o r -i
	address" arguments to remove it from the database and allow it
	to be used again. The machine_name argument defaults to the ip
	address if not explicitly named on the command line. The
	ip_address_name is a template for the ip address,
	e.g. "10.0.25.". If the file " filename" does not exist, then
	the first ip address is ip_address_name.1, as "10.0.25.1".

EOU
}

#
# Add an ip address/name to the database.
#
sub add_to_db(\%)
{
    my %argsHash=%{$_[0]};
    my $name=$argsHash{"name"};
    my $ipAddr=$argsHash{"ipAddr"};
    my %passHash = ("ipAddr" => $ipAddr);
    
    if (! query_db(%passHash))
    {
        $IPNAME{$ipAddr}=$name;
    }
}

#
# Is given ip record in database?
#
sub query_db(\%)
{
    no strict 'refs';

    my %argsHash=%{$_[0]};
    my $name=$argsHash{"name"};
    my $addr=$argsHash{"ipAddr"};
    my $retVal;

    if(! $name)
    {
        if($IPNAME{$addr})
        {
            $argsHash{"name"}=$IPNAME{$addr};
            $retVal=1;
        }
    }  
    elsif(! $addr)
    {
        my $nameKey;
        foreach $nameKey (keys %IPNAME)
        {
            if($IPNAME{$nameKey} =~ /^$name$/)
            {
                $argsHash{"ipAddr"}=$nameKey;
                $retVal=1;
            }
        }
    }
    else
    {
        if($IPNAME{$addr} !~ /^$name$/)
        {            
            $argsHash{"name"}="";
            $argsHash{"ipAddr"}="";
        }
    }
    %argsHash;
}

#
# Remove an ip/name pair from the database.
#
sub remove_from_db(\%)
{
    my %argsHash=%{$_[0]};
    my $name=$argsHash{"name"};
    my $ipAddr=$argsHash{"ipAddr"};
    
    # Find ipaddress by name if necessary.

    if((! $ipAddr) && ($name))
    {
        my $nameKey;
        foreach $nameKey (keys %IPNAME)
        {
            if($IPNAME{$nameKey} =~ /^$name$/)
            {
                $ipAddr=$nameKey;
            }
        }
    }

    if($ipAddr)
    {
        delete $IPNAME{$ipAddr};
    }
    return 1;
}

#
# Dump entire contents of ipaddr//name database to stdout.
#
sub dump_db()
{
    my $key;
    foreach $key (sort keys %IPNAME)
    {
        print("Name: ",$IPNAME{$key}," Address: ",$key,"\n");
    }
}

#
# Maintain (add/query/remove) the ip_address => machine name database
#
sub maintain_database()
{
    my %dbArgs=("name"=>"","ipAddr"=>"");
    my $retVal=1;
    

    chomp($opt_o);
    if((! $opt_o) || (length($opt_o) > 1) || ($opt_o !~ /a|r|q|d/) )
    {
        print("Operation must be one and only one of: a (add), r (remove), or q(query)\n");
        $retVal=0;
    }

    if($retVal == 1)
    {
        if( (! $opt_i) && (! $opt_n) && ($opt_o !~ /d/) )
        {
            print("Must have either a \"-i\" internet_address argument  or a \"n\" machine name  argument.");
            $retVal=0;
        }
    }
    # So much for argument parsing.

    if($retVal == 1)
    {
        if($opt_o =~ /a/)
        {
            if(! $opt_n)
            {
                $opt_n=$opt_i;
            }
            $dbArgs{"name"}=$opt_n;
            $dbArgs{"ipAddr"}=$opt_i;
            add_to_db(%dbArgs);
        }
        elsif($opt_o =~ /q/)
        {
            $dbArgs{"ipAddr"}=$opt_i;
            $dbArgs{"name"}=$opt_n;
            %dbArgs = query_db(%dbArgs);
            print("Name: ", $dbArgs{"name"}," Address: ",$dbArgs{"ipAddr"},"\n");
            
        } 
        elsif($opt_o =~ /r/)
        {
            $dbArgs{"ipAddr"}=$opt_i;
            if($opt_n)
            {
                $dbArgs{"name"}=$opt_n;
            }
            remove_from_db(%dbArgs);
        }
        elsif($opt_o =~ /d/)
        {
            dump_db();
        }
    }   

}

#
# Create a new ip address. Add it to the ip address database as needed.
#
sub make_ip_num()
{
    my $retVal=1;
    my $currentIP;
    my $ipStart=$opt_s;
    my %dbArgs=("name"=>"","ipAddr"=>"");

    if(! -f $opt_f)
    {
        open(OUTFILE,">$opt_f");
        $currentIP=$ipStart . "1";
        print(OUTFILE $currentIP);
        close(OUTFILE);
    }

    if (! open(INFILE,"<$opt_f"))
    { 
        print("Failing. Cannot open $opt_f: $!","\n");
        $retVal=0;
    }
    
    if($retVal)
    {        
        $dbArgs{"name"}=$opt_n;
        %dbArgs=query_db(%dbArgs);
        if($dbArgs{"ipAddr"})
        {
            #Oops. Machine name already exists in db. This is bad. 
            #Print current ip address for the name, then exit 1.
            print($dbArgs{"ipAddr"});            
            $retVal=0;
        }
    }

    if($retVal)
    {
        $currentIP = <INFILE>;
        close(INFILE);
        
        chomp($currentIP);
        $currentIP  =~ s/^$ipStart//; # 10.0.25.1 -> 1
        my $counter=0;       
        do
        {
            $counter++;
            ++$currentIP;
            if($currentIP < $opt_b)
            {
                # Underflow. Should never happen. Handle anyway.
                $currentIP = $opt_b;
            }
            if($currentIP  > $opt_t )
            {
                #Overflow -- start at beginning.               
                $currentIP=$opt_b;
                
            }            
            $dbArgs{"name"}="";
            $dbArgs{"ipAddr"}=$ipStart . $currentIP;
            %dbArgs=query_db(%dbArgs);

        } while ($dbArgs{"name"} && ($counter < 256) );

        
        if($counter < 255)
        {
            
            $currentIP=$ipStart . $currentIP;
            open(OUTFILE,">$opt_f");
            print(OUTFILE $currentIP);
            close(OUTFILE);

            $dbArgs{"ipAddr"}=$currentIP;
            if($opt_n) 
            {
                $dbArgs{"name"}=$opt_n;
            }
            else 
            {
                $dbArgs{"name"}=$currentIP;
            }
            add_to_db(%dbArgs);
            print($currentIP);
        } 
        else
        {
            print("Ouch! Out of ip addresses.\n");
            $retVal=0;
        }    
    }
    if($retVal == 1)
    {
        open(OUTFILE, ">$opt_f") || die ("Error writing $opt_f: $!");
        print(OUTFILE $currentIP);
        close(OUTFILE);
    }
    $retVal;
}

#
# Main Line
#

my $retVal=0;


if(! getopts("f:t:b:o:n:i:s:"))
{
    usage($0);
    die "Bad arguments.";
}

if (! $opt_t)
{
	$opt_t=254;
}

if (! $opt_f)
{
    $opt_f="./current.ip.num";
}

if(! $opt_b)
{
    $opt_b=1;
}

if(! $opt_s)
{
    $opt_s="10.0.25.";
}
my $dbDir=dirname($opt_f);
my $DBName="IpAddrName";

if(! dbmopen(%IPNAME,"$dbDir/$DBName",0666))
{
    die("Cannot open $dbDir/$DBName: $!");
}
my %dbArgs=("name"=>"","ipAddr"=>"");

if($opt_o)
{
    maintain_database();
}
else
{
    if(! make_ip_num())
    {
        $retVal=1;
    }
}

dbmclose(%IPNAME);

exit ($retVal);
